try:
    from pybase64 import b64decode, urlsafe_b64encode
except (ImportError, ModuleNotFoundError):
    from base64 import b64decode, urlsafe_b64encode
import binascii
import codecs
from functools import lru_cache
from hashlib import sha3_224

from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC


@lru_cache(32)
def fernet_key(base: str = '', salt: str = '') -> Fernet:
    """
    Generate Fernet key to encrypt / decrypt strings
    """

    if base and salt:
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA512(),
            length=32,
            salt=bytes(salt, encoding='utf8'),
            iterations=200000,
            backend=default_backend()
        )
        key = urlsafe_b64encode(kdf.derive(bytes(f'{base.upper()}_{base[::-1]}', encoding='utf8')))
    else:
        from .secrets import KEY
        key = binascii.unhexlify(b64decode(KEY)).swapcase()

    return Fernet(key)


def encrypt(string: str, key: Fernet = None) -> str:
    """
    Symmetrically encrypt strings to save in database.

    :param string: Alphanumeric string to encrypt
    :param key: Fernet Key, pre-generated for efficiency
    """
    if not key:
        key = fernet_key()

    try:
        byte_string = bytes(string, encoding='utf8')
    except TypeError:
        byte_string = string

    encrypted = key.encrypt(byte_string)

    return encrypted.decode(encoding='utf8')


def decrypt(string: str, key: Fernet = None) -> str:
    """
    Decrypt username/password string from database

    :param string: Alphanumeric string to decrypt
    :param key: Fernet Key, pre-generated for efficiency
    """
    if not key:
        key = fernet_key()

    try:
        byte_string = bytes(string, encoding='utf8')
    except TypeError:
        byte_string = string

    decrypted = key.decrypt(byte_string)

    return decrypted.decode(encoding='utf8')


def mask(string: str) -> str:
    """
    Mask / encode a string so that it's actual, useful form
    is not ever presented directly to human eyes.
    """
    value = codecs.encode(string[::-1], 'rot13').swapcase().encode()
    return urlsafe_b64encode(value).decode()


def unmask(string: str) -> str:
    """
    Reverse the process of masking a string.
    """
    try:
        value = b64decode(string).decode()[::-1]
    except binascii.Error:
        print('Value was not masked')
        return string
    return codecs.encode(value, 'rot13').swapcase()


def load_private_key(name: str):
    """
    Load private key prefix and return hashed value.

    Function used for bootstrapping new DrawBridge
    installations, when key is being retrieved,
    saved to disk and then loaded for use elsewhere.

    Use this function in the "elsewhere" call-site to
    read the value from disk.
    """
    from .configparser import SymmetricKeysConfig
    sym_keys = SymmetricKeysConfig().as_typed_dict()

    if name not in sym_keys:
        return ''

    return sha3_224(sym_keys.get(name, '').encode()).hexdigest()[12:48]


__all__ = (
    'decrypt',
    'encrypt',
    'fernet_key',
    'mask',
    'unmask',
    'load_private_key',
)
